package net.wanji.utc.dt.service.controller.impl;

import cn.hutool.core.util.HexUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.*;
import net.wanji.utc.dt.cache.CrossInfoCache;
import net.wanji.utc.dt.cache.SignalDataCache;
import net.wanji.utc.dt.common.constants.Constants;
import net.wanji.utc.dt.common.enums.ControlModelEnum;
import net.wanji.utc.dt.common.enums.SectionControlModeEnum;
import net.wanji.utc.dt.netty.NettyClient;
import net.wanji.utc.dt.pojo.dtconvert.PhaseInfoPojo;
import net.wanji.utc.dt.pojo.netty.MessageResultPojo;
import net.wanji.utc.dt.service.controller.ControlCommandService;
import net.wanji.utc.dt.util.CommonUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 10:43
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class DTControlCommandServiceImpl implements ControlCommandService {

    private final CrossInfoCache crossInfoCache;

    @Override
    public String getBeanMark() {
        return BaseEnum.VendorTypeEnum.DT.getNick();
    }

    /**
     * 东土相位灯组固定，万集下发相位就是下发东土阶段关联相位关系，
     * @param schemeSendVO
     * @return
     * @throws Exception
     */
    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        try {
            String crossId = schemeSendVO.getCrossCode();
            CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
            List<SchemeSendVO.Pattern> schemeVOList = schemeSendVO.getPatternList();
            setSchemePhaseStageInfo(crossInfoPO, schemeVOList);
        } catch (Exception e) {
            throw new Exception("下发方案异常", e);
        }
        return JsonViewObject.newInstance().success("下发方案成功");
    }

    private void setSchemePhaseStageInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {
        setPhaseStageInfo(crossInfoPO, schemeVOList);
        setSchemeInfo(crossInfoPO, schemeVOList);
    }

    private void setSchemeInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {
        StringBuffer sb = new StringBuffer();
        for (SchemeSendVO.Pattern schemeVO : schemeVOList) {
            String schemeNo = schemeVO.getPatternNo();
            String cycle = schemeVO.getCycle();
            String offset = schemeVO.getOffset();

            String matchStr = "0902%s%s";
            for (int i = 2; i <= 8; i++) {
                sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", Integer.valueOf(schemeNo))));
                if (i == 2) {
                    sb.append("01").append(String.format("%02x", i)).append("08"); //路口序号 + 序号
                }
                if (i == 3) {
                    String cycleHex = StringUtils.leftPad(String.format("%02x", Integer.valueOf(cycle)), 8, "0");
                    sb.append(cycleHex).append(String.format("%02x", i)).append("05");
                }
                if (i == 4) { //协调序号 默认00
                    sb.append("00").append(String.format("%02x", i)).append("06");
                }
                if (i == 5) {
                    String offsetHex = StringUtils.leftPad(String.format("%02x", Integer.valueOf(offset)), 4, "0");
                    sb.append(offsetHex).append(String.format("%02x", i)).append("06");
                }
                if (i == 6) {
                    String phaseStageIdChain = getPhaseStageIdChainHex(schemeVO.getRings());
                    sb.append(phaseStageIdChain).append(String.format("%02x", i)).append("24");
                }
                if (i == 7) {
                    String phaseStageIdChain = getPhaseStageIdTimeChainHex(schemeVO.getRings());
                    sb.append(phaseStageIdChain).append(String.format("%02x", i)).append("14");
                }
                if (i == 8) {
                    String phaseStageIdTypeChain = getPhaseStageIdTypeChainHex(schemeVO.getRings());
                    sb.append(phaseStageIdTypeChain).append(String.format("%02x", i)).append("05");
                }
            }
        }

        String bodyValueLength = String.format("%02x", schemeVOList.size() * 7); // 调度数量 * 属性数量
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLength, sb.substring(0, sb.length() - 4));
        String schemeSign = StringUtils.join(Constants.COMMAND_SCHME_CONFIG, Constants.COMMAND_SET_REPLY);
        String schemeMessage = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), schemeMessage, schemeSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("下发方案信息成功");
        }
    }

    private String getPhaseStageIdTypeChainHex(List<SchemeSendVO.Pattern.Ring> rings) {
        StringBuffer sb = new StringBuffer();
        for (SchemeSendVO.Pattern.Ring ring : rings) { // 都是单环模式 相位就是阶段
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
            for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                sb.insert(0, "01");
            }
        }
        return StringUtils.leftPad(sb.toString(), 32, "0");
    }

    private String getPhaseStageIdTimeChainHex(List<SchemeSendVO.Pattern.Ring> rings) {
        StringBuffer sb = new StringBuffer();
        for (SchemeSendVO.Pattern.Ring ring : rings) { // 都是单环模式 相位就是阶段
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
            for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                String phaseTime = phase.getPhaseTime();
                sb.insert(0, String.format("%02x", Integer.valueOf(phaseTime)));
            }
        }
        return StringUtils.leftPad(sb.toString(), 64, "0");
    }

    private String getPhaseStageIdChainHex(List<SchemeSendVO.Pattern.Ring> rings) {
        StringBuffer sb = new StringBuffer();
        for (SchemeSendVO.Pattern.Ring ring : rings) { // 都是单环模式 相位就是阶段
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
            for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                String phaseNo = phase.getPhaseNo();
                sb.insert(0, String.format("%02x", Integer.valueOf(phaseNo)));
            }
        }
        return StringUtils.leftPad(sb.toString(), 32, "0");
    }

    /**
     * 下发阶段，相位灯组固定，阶段内关联相位，相位1就是北左转
     * @param crossInfoPO
     * @param schemeVOList
     */
    private void setPhaseStageInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {
        StringBuffer sb = new StringBuffer();
        int phaseStageSize = 0;
        String matchStr = "0602%s%s";
        for (SchemeSendVO.Pattern pattern : schemeVOList) {
            List<SchemeSendVO.Pattern.Ring> rings = pattern.getRings();
            for (SchemeSendVO.Pattern.Ring ring : rings) { // 单环
                List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
                phaseStageSize = phaseList.size();
                for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                    Integer phaseNo = Integer.valueOf(phase.getPhaseNo());
                    for (int i = 2; i <= 4; i++) {
                        sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", phaseNo)));
                        if (i == 2) {
                            sb.append(StringUtils.leftPad(HexUtil.toHex(CommonUtils.get4BinaryInteger(phaseNo)), 16, "0"));
                            sb.append(String.format("%02x", i)).append("44");
                        }
                        if (i == 3) {
                            sb.append("00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000");
                            sb.append(String.format("%02x", i)).append("44");
                        }
                        if (i == 4) {
                            sb.append("00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000");
                            sb.append(String.format("%02x", i)).append("0c");
                        }
                    }
                }
            }
        }

        String bodyValueLength = String.format("%02x", phaseStageSize * 4);
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLength, sb.substring(0, sb.length() - 4));
        String setPhaseStageSign = StringUtils.join(Constants.COMMAND_PHASE_STAGE_CONFIG, Constants.COMMAND_SET_REPLY);
        String setPhaseStageMessage = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), setPhaseStageMessage, setPhaseStageSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("下发相位阶段成功");
        }
    }


    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        String crossId = planSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<PlanSendVO.Plan> planList = planSendVO.getPlanList();
        StringBuilder sb = new StringBuilder();
        for (PlanSendVO.Plan plan : planList) {
            sb.append(buildDailyPlanHex(plan));
        }
        // 只下发有效计划返回报错
        if (planList.size() < 8) {
            for (int i = planList.size() + 1; i <= 8 - planList.size(); i++) {
                PlanSendVO.Plan plan = new PlanSendVO.Plan();
                plan.setPlanNo(String.valueOf(i));
                sb.append(buildDailyPlanHex(plan));
            }
        }
        String bodyValueLength = String.format("%02x", planList.size() * 5); // 调度数量 * 属性数量
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLength, sb.substring(0, sb.length() - 4));
        String planSign = StringUtils.join(Constants.COMMAND_DAILYPLAN_CONFIG, Constants.COMMAND_SET_REPLY);
        String planMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), planMessageHex, planSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            return JsonViewObject.newInstance().success("下发计划成功");
        }
        return JsonViewObject.newInstance().success("下发计划异常");
    }

    private String buildDailyPlanHex(PlanSendVO.Plan plan) {
        String planNo = plan.getPlanNo();
        List<PlanSendVO.Plan.Section> sectionList = plan.getSectionList();
        StringBuilder hourMinuteSb = new StringBuilder();
        StringBuilder schemeNoSb = new StringBuilder();
        StringBuilder controlModelSb = new StringBuilder();
        if (!CollectionUtils.isEmpty(sectionList)) {
            for (PlanSendVO.Plan.Section section : sectionList) {
                String schemeNo = section.getPatternNo();
                String schemeHex = String.format("%02x", Integer.valueOf(schemeNo));
                schemeNoSb.insert(0, schemeHex);
                String beginTime = section.getBeginTime();
                String startHourMinuteHex = CommonUtils.getHourMinuteHex(beginTime);
                String endTime = section.getEndTime();
                String endHourMinuteHex = CommonUtils.getHourMinuteHex(endTime);
                String controlMode = section.getControlMode();
                String dtControlMode = SectionControlModeEnum.getDtControlMode(Integer.valueOf(controlMode));
                // 去除重复时段控制模式 万集会有重复时段，重复则不插入当前时段
                if (!hourMinuteSb.toString().contains(startHourMinuteHex)) {
                    schemeNoSb.insert(0, schemeHex);
                    hourMinuteSb.insert(0, startHourMinuteHex);
                    controlModelSb.insert(0, dtControlMode);
                }
                controlModelSb.insert(0, dtControlMode);
                hourMinuteSb.insert(0, endHourMinuteHex);
            }
        }
        String matchStr = "0b02%s%s";
        StringBuffer sb = new StringBuffer();
        for (int i = 2; i <= 5; i++) { // 属性
            sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", Integer.parseInt(planNo))));
            if (i == 2) {
                sb.append("01").append(String.format("%02x", i)).append("64");
            }
            if (i == 3) {
                sb.append(StringUtils.leftPad(hourMinuteSb.toString(), 192, "0")).append(String.format("%02x", i)).append("34");
            }
            if (i == 4) {
                sb.append(StringUtils.leftPad(schemeNoSb.toString(), 96, "0")).append(String.format("%02x", i)).append("34");
            }
            if (i == 5) {
                sb.append(StringUtils.leftPad(controlModelSb.toString(), 96, "0")).append(String.format("%02x", i)).append("05");
            }
        }
        return sb.toString();
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        String crossId = scheduleSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<ScheduleSendVO.Schedule> schedules = scheduleSendVO.getSchedules();
        StringBuilder sb = new StringBuilder();
        for (ScheduleSendVO.Schedule schedule : schedules) {
            sb.append(buildScheduleHex(schedule));
        }
        String bodyValueLeght = String.format("%02x", schedules.size() * 6); // 调度数量 * 属性数量
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLeght, sb.toString());
        String scheduleSign = StringUtils.join("0901", Constants.COMMAND_SET_REPLY);
        String scheduleMessage = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), scheduleMessage, scheduleSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            return JsonViewObject.newInstance().success("下发调度成功");
        }
        return JsonViewObject.newInstance().success("下发调度异常");
    }

    /**
     * 东土调度必须同时有星期和月日类型
     *
     * @param schedule
     * @return
     */
    private String buildScheduleHex(ScheduleSendVO.Schedule schedule) {
        String scheduleNo = schedule.getScheduleNo();
        Integer planNo = 0;
        String weekHex = "00";
        List<ScheduleSendVO.Schedule.Week> weeks = schedule.getWeeks();
        if (!CollectionUtils.isEmpty(weeks)) {
            List<ScheduleSendVO.Schedule.Week> weekSort = weeks.stream().sorted(Comparator.comparingInt(ScheduleSendVO.Schedule.Week::getWeekNum)).collect(Collectors.toList());
            int[] weekArray = new int[7];
            for (ScheduleSendVO.Schedule.Week week : weekSort) {
                Integer weekNum = week.getWeekNum();
                weekArray[weekNum - 1] = 1;
                planNo = week.getPlanNo();
            }
            weekHex = CommonUtils.getHex4Array(weekArray);
        }
        String monthHex = "0000";
        String dayHex = "00000000";
        List<ScheduleSendVO.Schedule.SpecialDay> specialDays = schedule.getSpecialDays();
        if (!CollectionUtils.isEmpty(specialDays)) {
            int[] monthArray = new int[12];
            int[] dayArray = new int[31];
            for (ScheduleSendVO.Schedule.SpecialDay specialDay : specialDays) {
                String dateStr = specialDay.getDateStr();
                monthArray[Integer.valueOf(dateStr.substring(5, 7)) - 1] = 1;
                dayArray[Integer.valueOf(dateStr.substring(8, 10)) - 1] = 1;
            }
            monthHex = CommonUtils.getHex4Array(monthArray);
            dayHex = CommonUtils.getHex4Array(dayArray);
        }
        String matchStr = "0c02%s%s";
        StringBuffer sb = new StringBuffer();
        for (int i = 2; i < 8; i++) { // 数据对象从2-7
            sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", Integer.valueOf(scheduleNo))));
            if (i == 2) {
                sb.append("01");// 一代多路口编号
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 3) {
                sb.append("00"); // 优先级
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 4) {
                sb.append(weekHex); // 星期二进制转16
                sb.append(String.format("%02x", i)).append("06");
            }
            // 如果星期类型位1-7，则日月全部下发
            if (StringUtils.equals(weekHex, "7f")) {
                monthHex = "0fff";
                dayHex = "7fffffff";
            }
            if (i == 5) {
                sb.append(StringUtils.leftPad(monthHex, 4, "0"));
                sb.append(String.format("%02x", i)).append("08");
            }
            if (i == 6) {
                sb.append(StringUtils.leftPad(dayHex, 8, "0"));
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 7) {
                sb.append(String.format("%02x", planNo));
                sb.append(String.format("%02x", i)).append("05");
            }
        }
        return sb.toString().substring(0, sb.length() - 4);
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        if (commandVO.getCommand() == 0) {
            setSignalControl(commandVO);
        } else {
            List<Integer> phaseList = commandVO.getPhaseList();
            int stageId = 0;
            if (!CollectionUtils.isEmpty(phaseList)) {
                stageId = phaseList.get(0);
            }  // 锁定阶段，目前锁定成功，但是阶段还在跳
            CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(commandVO.getCrossCode());
            String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_STAGE, String.format("%02x", stageId));
            String lockSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
            String lockMessage = Constants.buildMessage(message);
            MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), lockMessage, lockSign, 300);
            if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
                log.info("锁定相位成功");
            }
        }

        return JsonViewObject.newInstance().success("锁定相位成功");
    }

    @Override
    public JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject setSignalControl(ControlCommandVO commandVO) throws Exception {
        String controlmodel = null;
        if (commandVO.getCommand() == 0) {
            controlmodel = ControlModelEnum.FIXED_CYCLE.getDtControlHex(); // 取消控制，设置为固定配时
        } else {
            if (commandVO.getControlType() == 1) { // 全红
                controlmodel = ControlModelEnum.RED_CONTROL.getDtControlHex();
            }
            if (commandVO.getControlType() == 2) { // 黄闪
                controlmodel = ControlModelEnum.YELLOW_CONTROL.getDtControlHex();
            }
            if (commandVO.getControlType() == 3) { // 关灯
                controlmodel = ControlModelEnum.CLOSED_CONTROL.getDtControlHex();
            }
        }
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(commandVO.getCrossCode());
        String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_MODEL, controlmodel);
        String controlSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
        String controlMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), controlMessageHex, controlSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("特殊控制成功，当前控制模式： {}", controlmodel);
        }
        return JsonViewObject.newInstance().success("特殊控制成功");
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(code);
        String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_MODEL, ControlModelEnum.FIXED_CYCLE.getDtControlHex());
        String controlSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
        String controlMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), controlMessageHex, controlSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("恢复时间表");
        }
        return JsonViewObject.newInstance().success("恢复时间表");
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String crossId = phaseTimingSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<PhaseTimingSendVO.Phase> phaseList = phaseTimingSendVO.getPhaseList();
        setLights();
        setPhaseLights();
        setPhaseInfo(crossInfoPO, phaseList);
        return JsonViewObject.newInstance().success();
    }

    private void setPhaseLights() {

    }

    private void setLights() {
    }

    /**
     * 东土相位灯组固定，只修改相位绿灯，绿闪，红灯，黄灯，最大最小绿时间
     * @param crossInfoPO
     * @param phaseList
     */
    private void setPhaseInfo(CrossInfoPO crossInfoPO, List<PhaseTimingSendVO.Phase> phaseList) {
        String matchStr = "0402%s%s";
        StringBuffer sb = new StringBuffer();
        for (int j = 1; j <= 16; j++) {
            Integer yellow = 0;
            Integer allRed = 0;
            Integer greenFlash = 0;
            Integer maxGreen = 0;
            Integer minGreen = 0;
            String phaseNoHex = String.format("%02x", j);
            // 下发相位数据
            for (PhaseTimingSendVO.Phase phase : phaseList) { // 绿灯时间 = 阶段时间 - 黄灯 - 红灯  22 - 2 - 3
                Integer phaseNo = phase.getPhaseNo();
                if (j == phaseNo) {
                    yellow = phase.getYellow();
                    allRed = phase.getAllred();
                    greenFlash = Integer.valueOf(phase.getGreenFlash());
                    maxGreen = Integer.valueOf(phase.getMaxGreen());
                    minGreen = Integer.valueOf(phase.getMinGreen());
                }
            }
            // 原东土相位数据
            List<PhaseInfoPojo> phaseInfoList = SignalDataCache.phaseInfoCache.get(crossInfoPO.getId());
            PhaseInfoPojo phaseInfoPojo = null;
            for (PhaseInfoPojo element : phaseInfoList) {
                if (Objects.equals(j, element.getPhaseId())) {
                    phaseInfoPojo = element;
                }
            }
            // 如果时间为0， 使用东土原数据
            yellow = yellow == 0 ? phaseInfoPojo.getYellowTime() : yellow;
            allRed = allRed == 0 ? phaseInfoPojo.getRedTime() : allRed;
            greenFlash = greenFlash == 0 ? phaseInfoPojo.getGreenFlashTime() : greenFlash;
            maxGreen = maxGreen == 0 ? phaseInfoPojo.getMaxGreenTime() : maxGreen;
            minGreen = minGreen == 0 ? phaseInfoPojo.getMinGreenTime() : minGreen;
            for (int i = 2; i <= 31; i++) {
                if (i < 11 || (i > 26 && i < 30)) {
                    sb.append(String.format(matchStr, String.format("%02x", i), phaseNoHex));
                }
                if (i == 2) {
                    String phaseHex = StringUtils.leftPad(CommonUtils.getBinary(j), 16, "0");
                    sb.append(phaseHex).append(String.format("%02x", i)).append("05");
                }
                if (i == 3) {
                    sb.append("20").append(String.format("%02x", i)).append("05"); // 绿灯类型
                }
                if (i == 4) {
                    Integer greenTime = phaseInfoPojo.getGreenTime();
                    sb.append(String.format("%02x", greenTime)).append(String.format("%02x", i)).append("05"); // 绿灯时间
                }
                if (i == 5) {
                    sb.append("21").append(String.format("%02x", i)).append("05"); // 绿闪类型
                }
                if (i == 6) {
                    sb.append(String.format("%02x", Integer.valueOf(greenFlash))).append(String.format("%02x", i)).append("05"); // 绿闪时间
                }
                if (i == 7) {
                    sb.append("30").append(String.format("%02x", i)).append("05"); // 黄灯类型
                }
                if (i == 8) {
                    sb.append(String.format("%02x", Integer.valueOf(yellow))).append(String.format("%02x", i)).append("05"); // 绿闪时间
                }
                if (i == 9) {
                    sb.append("10").append(String.format("%02x", i)).append("05"); // 红灯类型
                }
                if (i == 10) {
                    Integer redTime = Integer.valueOf(allRed) + Integer.valueOf(yellow) + Integer.valueOf(greenFlash) + phaseInfoPojo.getGreenTime();
                    sb.append(String.format("%02x", redTime)).append(String.format("%02x", i)).append("05"); // 红灯时间
                }
                if (i == 11) {
                    String str = "04020B%s000B0504020C%s000C0504020D%s000D0504020E%s000E0504020F%s310F05040210%s0A1005040211%s001105040212%s001205040213%s001305040214%s001405040215%s311505040216%s0A1605040217%s001705040218%s001805040219%s00190504021A%s001A06";
                    sb.append(str.replaceAll("%s", phaseNoHex));
                }
                if (i > 11 && i <= 26) {
                    continue;
                }
                if (i == 27) {
                    String minGreenHex = StringUtils.leftPad(String.format("%02x", Integer.valueOf(minGreen)), 4, "0");
                    sb.append(minGreenHex).append(String.format("%02x", i)).append("06");
                }
                if (i == 28 || i == 29) {
                    String maxGreenHex = StringUtils.leftPad(String.format("%02x", Integer.valueOf(maxGreen)), 4, "0");
                    sb.append(maxGreenHex).append(String.format("%02x", i)).append("06");
                }
                if (i == 30) {
                    sb.append(String.format("04021E%s00031E0C", phaseNoHex)); // 延长绿时间
                }
                if (i == 31) {
                    sb.append(String.format("04021F%s00000000000000001F0C", phaseNoHex));
                }
            }
        }

        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, "ff", sb.substring(0, sb.length() - 4));
        String phaseSign = StringUtils.join(Constants.COMMAND_SCHME_CONFIG, Constants.COMMAND_SET_REPLY);
        String phaseMessage = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), phaseMessage, phaseSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("下发相位成功");
        }
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }

}
